package weka.active;

import java.util.*;
import java.io.*;
import weka.core.*;
import weka.active.*;

public class ActiveEvaluationPlotter
{
    private static int m_seed = 1; 
    private static int m_numRuns = 1;
    public static int m_currRun = 0;
    
    public static String evaluateModelAndPlot(ActiveLearner learner, String[] options)
        throws Exception
    {
        int i,j;
        FastVector evalResults = new FastVector();
        Random random;
        String[] optionsCopy;
        StringBuffer matlab = new StringBuffer();
        StringBuffer evalStrings = new StringBuffer();
        ActiveLearner[] learners = new ActiveLearner[m_numRuns];
        String[][] optionsCopies = new String[m_numRuns][];
                        
        ActiveEvaluation.setDoNotWriteToFile(true);
        if (learner instanceof RandomUser) {            
            random = new Random(m_seed);
            
            for(i = 0; i < learners.length; i++) {
                learners[i] = (ActiveLearner)(learner.getClass().newInstance());
                optionsCopies[i] = new String[options.length];
                for (j=0; j<options.length; j++) {
                    optionsCopies[i][j] = new String(options[j]);
                }            
            }
            for (i=0; i<m_numRuns; i++) {                
                ((RandomUser)learners[i]).setSeed(random.nextInt());
                m_currRun = i;
                evalStrings.append("\n" + "----------------------------------\n" + 
                                   ActiveEvaluation.evaluateModel(learners[i], optionsCopies[i]));
                evalResults.addElement(ActiveEvaluation.getLastRunResult());                
            }
        } else {
            evalStrings.append(ActiveEvaluation.evaluateModel(learner, options));
            evalResults.addElement(ActiveEvaluation.getLastRunResult());            
        }                
        ActiveEvaluation.setDoNotWriteToFile(false);
        
        FileOutputStream outFileMatlab = new FileOutputStream(ActiveEvaluation.getLastOutFileName() + ".m");                
        FileOutputStream outFileString = new FileOutputStream(ActiveEvaluation.getLastOutFileName() + ".txt");
        outFileString.write(evalStrings.toString().getBytes());
        outFileString.close();
        
        Double d;              
        int runCount=0, foldCount=0, queryCount=0;
        int numRuns = evalResults.size();
        int numQueries, numFolds;        
        ActiveEvaluation[] oneRunQueryResult;
        FastVector[][] totalFoldResults = new FastVector[numRuns][];
        FastVector oneQueryResult, oneFoldResult;
        FastVector xVec = new FastVector();        
        Enumeration doubleEnum;        
        // init         
        oneRunQueryResult = (ActiveEvaluation[])evalResults.elementAt(0);
        oneQueryResult = oneRunQueryResult[0].getPctCorrectVec();
        numFolds = oneQueryResult.size();
        numQueries = oneRunQueryResult.length;        
        for (runCount=0; runCount<numRuns; runCount++) {
            totalFoldResults[runCount] = new FastVector[numFolds];
            for (foldCount=0; foldCount<numFolds; foldCount++) {
                totalFoldResults[runCount][foldCount] = new FastVector();
            }
        }
        // collect fold results and output query results
        for (queryCount=0; queryCount < numQueries; queryCount++) {
            if (oneRunQueryResult[queryCount] == null) {          
                queryCount++;
                continue;
            }            
            xVec.addElement(new Integer(queryCount));
            matlab.append("query_" + queryCount + " = [\n");            
            for (runCount=0; runCount<numRuns; runCount++) {
                oneRunQueryResult = (ActiveEvaluation[])evalResults.elementAt(runCount);                
                oneQueryResult = oneRunQueryResult[queryCount].getPctCorrectVec();
                doubleEnum = oneQueryResult.elements();
                foldCount=0; // count folds
                while (doubleEnum.hasMoreElements()) {
                    d = (Double)doubleEnum.nextElement();
                    matlab.append("" + Utils.doubleToString(d.doubleValue(), 10, 4) + "\n");                    
                    totalFoldResults[runCount][foldCount].addElement(new Double(d.doubleValue()));
                    foldCount++;
                }                                
            }
            matlab.append(" ];\n");            
        } 
        // output fold results
        for (runCount=0; runCount<numRuns; runCount++) {
            for (foldCount=0; foldCount<numFolds; foldCount++) {
                matlab.append("fold_" + runCount + "_" + foldCount + "= [\n");                            
                doubleEnum = totalFoldResults[runCount][foldCount].elements();
                while (doubleEnum.hasMoreElements()) {
                    d = (Double)doubleEnum.nextElement();
                    matlab.append("" + Utils.doubleToString(d.doubleValue(), 10, 4) + "\n");                    
                }  
                matlab.append(" ];\n");                            
            }
        }
        matlab.append(outputMatlabProgram(learner.getClass().getName(), xVec, numFolds));
        outFileMatlab.write(matlab.toString().getBytes());
        outFileMatlab.close();                    
        return evalStrings.toString();                            
    }
    
    private static String outputMatlabProgram(String className, FastVector xVec, int numFolds)
    {
        StringBuffer text = new StringBuffer();
        Enumeration indxEnum;
        Integer x;
        
        text.append("\n\n");
        indxEnum = xVec.elements();
        while (indxEnum.hasMoreElements()) {
            x = (Integer)indxEnum.nextElement();
            text.append("query_" + x.intValue() + "_avg = mean(query_" + x.intValue() + ");\n");
            text.append("query_" + x.intValue() + "_sd = (std(query_" + x.intValue() + ") / sqrt(" + numFolds + "));\n"); 
        }
        text.append("\nX_vector = [\n");
        indxEnum = xVec.elements();
        while (indxEnum.hasMoreElements()) {
            x = (Integer)indxEnum.nextElement();
            text.append("\t" + x.intValue() + " \n");
        }
        text.append("];\n");
        text.append("avg_vec = [\n");
        indxEnum = xVec.elements();
        while (indxEnum.hasMoreElements()) {
            x = (Integer)indxEnum.nextElement();
            text.append("\tquery_" + x.intValue() + "_avg \n");
        }
        text.append("];\n");
        text.append("sd_vec = [\n");
        indxEnum = xVec.elements();
        while (indxEnum.hasMoreElements()) {
            x = (Integer)indxEnum.nextElement();
            text.append("\tquery_" + x.intValue() + "_sd \n");
        }
        text.append("];\n");
        text.append("figure(1);\n");
        //text.append("handle = "); // for legend
        text.append("errorbar(X_vector, avg_vec, sd_vec);\n");
        //text.append("h = handle(2);\n"); // for legend
        text.append("title('" + className + "');\n" +
                    "xlabel('size of training set');\nylabel('pct on testing set');\n");
        text.append("axis auto;\nhold on;\n");
        return text.toString();        
    }
    
    public static void Plot(FastVector oneRunResult, String suffix)
        throws Exception
    {
        int i,j;                
        StringBuffer matlab = new StringBuffer();                                               
        FileOutputStream outFileMatlab = new FileOutputStream(ActiveEvaluation.getLastOutFileName() + "_" + suffix + ".m");                                
        Double d;                      
        int foldCount=0, queryCount=0;        
        int numQueries, numFolds;                
        FastVector[] totalFoldResults;
        FastVector oneQueryResult, oneFoldResult;
        FastVector xVec = new FastVector();        
        Enumeration doubleEnum;        
        // init                 
        oneQueryResult = ((FastVector)oneRunResult.elementAt(2));      
        numFolds = oneQueryResult.size();
        numQueries = oneRunResult.size();        
        totalFoldResults = new FastVector[numFolds];
        for (foldCount=0; foldCount<numFolds; foldCount++) {
            totalFoldResults[foldCount] = new FastVector();
        }        
        // collect fold results and output query results
        for (queryCount=0; queryCount < numQueries; queryCount++) {
            oneQueryResult = (FastVector)(oneRunResult.elementAt(queryCount));
            if (oneQueryResult.size()==0) {          
                queryCount++;
                continue;
            }            
            xVec.addElement(new Integer(queryCount));
            matlab.append("query_" + queryCount + " = [\n");            
            doubleEnum = oneQueryResult.elements();
            foldCount=0; // count folds
            while (doubleEnum.hasMoreElements()) {
                d = (Double)doubleEnum.nextElement();
                matlab.append("" + Utils.doubleToString(d.doubleValue(), 10, 4) + "\n");                    
                totalFoldResults[foldCount].addElement(new Double(d.doubleValue()));
                foldCount++;                                                
            }
            matlab.append(" ];\n");            
        } 
        // output fold results
        for (foldCount=0; foldCount<numFolds; foldCount++) {
            matlab.append("fold_0_" + foldCount + "= [\n");                            
            doubleEnum = totalFoldResults[foldCount].elements();
            while (doubleEnum.hasMoreElements()) {
                d = (Double)doubleEnum.nextElement();
                matlab.append("" + Utils.doubleToString(d.doubleValue(), 10, 4) + "\n");                    
            }  
            matlab.append(" ];\n");                                        
        }
        matlab.append(outputMatlabProgram("selfTest", xVec, numFolds));
        outFileMatlab.write(matlab.toString().getBytes());
        outFileMatlab.close();                            
    }    
}