package weka.active;

import java.io.Serializable;
import weka.core.*;

public abstract class ActiveLearner implements Serializable
{
    protected final int NO_CLASS = -1;
    
    public abstract void startActiveLearningSession() throws Exception;    
    public abstract void setDataAndPool(Instances data, DataContainer pool) throws Exception;
    public abstract void buildClassifier(DataContainer data) throws Exception;
    public abstract double classifyInstance(InstanceContainer instance) throws Exception; 
    // returns the index of the instance to query
    public abstract int instanceToQuery(DataContainer pool) throws Exception;
    
    public static ActiveLearner forName(String learnerName, String [] options) throws Exception {
        return (ActiveLearner)Utils.forName(ActiveLearner.class,
				                            learnerName,
				                            options);
    }
    
    public static ActiveLearner[] makeCopies(ActiveLearner model,
					                       int num) throws Exception {
        if (model == null) {
            throw new Exception("No model classifier set");
        }
        ActiveLearner[] learners = new ActiveLearner[num];
        String [] options = null;
        if (model instanceof OptionHandler) {
            options = ((OptionHandler)model).getOptions();
        }
        for(int i = 0; i < learners.length; i++) {
            learners[i] = (ActiveLearner) model.getClass().newInstance();
            if (options != null) {
	            String [] tempOptions = (String [])options.clone();
	            ((OptionHandler)learners[i]).setOptions(tempOptions);
	            Utils.checkForRemainingOptions(tempOptions);
            }
        }
        return learners;
    }            
}