package weka.active;

import java.util.*;
import weka.active.*;
import weka.core.*;

class MABExp4 
{    
    public int m_numExperts = 0;    
    public double[] m_expertProb = null;
    public double[] m_expertUnnormProbs = null;
    public int m_lastActionChoise = -1;
    public double m_lastActionProb = 0.0;
    public double[] m_lastActionProbArr = null;
    public Random m_rndm = null;    
    private double m_alpha = 0.5;
    private double m_gamma = 0.5;        
    
    public MABExp4() 
    {}

    public void start(int numExperts,
                      Random rndm)
    {
        int i;        
        this.m_lastActionChoise = -1;
        this.m_lastActionProb = 0.0;
        this.m_rndm = rndm;              
        this.m_numExperts = numExperts;        
        this.m_expertProb = new double[numExperts];
        this.m_expertUnnormProbs = new double[numExperts];
        for (i=0; i<numExperts; i++) {                               
            this.m_expertProb[i] = 1.0 / ((double)numExperts);
        }
        for (i=0; i<numExperts; i++) {                        
            this.m_expertUnnormProbs[i] = 1;
        }
        // gamma and alpha are calculated each round because number of actions is dynamic 
    }
    
    public int chooseAction(double[][] adviceProbs, double GLimit, boolean rndmFlag) throws Exception
    {   
        if (adviceProbs.length != this.m_numExperts) throw new Exception("wrong number of experts");
        int i, j;
        double sumProbs;
        double sumPerAction;
        double[] actionsProb = new double[adviceProbs[0].length];
        int numActions = actionsProb.length;
        // calculate gamma and alpha for this round 
        this.m_gamma = this.getBestGamma(GLimit, 
                                         numActions); 
        this.m_alpha = this.getBestAlpha(GLimit, 
                                         this.m_gamma, 
                                         numActions);
        // print 
        System.out.println("ChooseAction: gamma=" + this.m_gamma + " alpha=" + this.m_alpha);
        System.out.println("Probs of experts: ");
        for (i=0; i<this.m_numExperts; i++) {
            System.out.println(" prob[" + i + "]=" + this.m_expertProb[i]);
        }
        System.out.println("");
        
        sumProbs = 0.0;
        for (i=0; i<numActions; i++) {            
            sumPerAction = 0.0;
            for (j=0; j<this.m_numExperts; j++) {
                sumPerAction += this.m_expertProb[j] * adviceProbs[j][i];
            }
            actionsProb[i] = ((1-this.m_gamma) * sumPerAction) + (this.m_gamma * (1.0/numActions));            
            sumProbs += actionsProb[i];
        }
        this.normalize(actionsProb, 
                       actionsProb, 
                       sumProbs, 
                       false);
        
        this.m_lastActionChoise = this.chooseWithProb(actionsProb, rndmFlag);
        this.m_lastActionProb = actionsProb[this.m_lastActionChoise];
        this.m_lastActionProbArr = actionsProb;
        return this.m_lastActionChoise;        
    }
    
    public void updateReward(double reward, 
                             double[] chosenProbs) throws Exception
    {
        int i;        
        double sum;
                
        if (this.m_lastActionChoise == -1) return;        
        double[] rewards = new double[this.m_numExperts];
        for (i=0; i<rewards.length; i++) {
            rewards[i] = (reward / this.m_lastActionProb) * chosenProbs[i];  
        }        
        sum = 0.0;
        for (i=0; i<this.m_numExperts; i++) {
            this.m_expertUnnormProbs[i] *= Math.exp(m_alpha * rewards[i]);                        
            if (Double.isInfinite(this.m_expertUnnormProbs[i])) throw new Exception("Infinite prob");         
            sum += this.m_expertUnnormProbs[i];
            if (Double.isInfinite(sum)) throw new Exception("Infinite sum prob");
        }                         
        this.normalize(this.m_expertProb, 
                       this.m_expertUnnormProbs, 
                       sum, 
                       false);          
    }    
        
    public int chooseWithProb(double[] prob, boolean rndmFlag) throws Exception
    {
        int i;        
        
        if (rndmFlag) {
            double r = this.m_rndm.nextDouble();
            double sum = 0.0;
            for (i=0; i<prob.length; i++) {   
                sum += prob[i];
                if (Utils.grOrEq(sum, r)) {
                    return i;
                }            
            }            
            throw new Exception("failed chooseWithProb");
        } else {                
            double maxProb = -Double.MAX_VALUE;
            int maxProbIndx = -1; 
            for(i=0; i<prob.length; i++) {               
                if (Utils.gr(prob[i], maxProb)) {
                    maxProb = prob[i];
                    maxProbIndx = i;
                }
            }
            if (maxProbIndx == -1) throw new Exception("failed chooseWithProb");        
            return maxProbIndx;        
        }        
    }
          
    public void normalize(double[] dst, 
                          double[] src, 
                          double sum, 
                          boolean printFlag) throws Exception
    {
        int i;
        int l = dst.length;
        if (l != src.length) throw new Exception("length src dst no equal");
        for (i=0; i<l; i++) {
            dst[i] = src[i] / sum;
        }
        sum = 0.0;
        if (printFlag) {
            System.out.println(" ");
        }
        for (i=0; i<l; i++) {
            if (printFlag) {
                System.out.print(" prob[" + i + "]=" + dst[i]);
            }
            sum += dst[i];
        }
        if (printFlag) {
            System.out.println(" ");
        }
        if (!Utils.eq(sum, 1.0)) throw new Exception("sum of prob is not 1 sum=" + sum);
    }
                                       
    public void setAlpha(double alpha) 
    {
        this.m_alpha = alpha;
    }
    
    public double getAlpha()
    {
        return m_alpha;
    }
            
    public void setGamma(double gamma)
    {
        this.m_gamma = gamma;
    }
    
    public double getGamma()
    {
        return this.m_gamma;
    }
        
    public double getBestAlpha(double GLimit, 
                               double bestGamma, 
                               int numActions)
    {
        return (bestGamma / ((double)numActions)); 
    }
    
    public double getBestGamma(double GLimit, 
                               int numActions)
    {
        double k = (double)numActions;     
        double n = (double)this.m_numExperts;
        double r = Math.sqrt((k * Math.log(n)) / ((Math.E-1.0) * GLimit));
        if (Utils.gr(r, 1.0)) {
            r = 1.0;
        }
        return r;
    }                 
}

public class MultiArmBandit2
{
    private MABExp4 m_algo;
    private double[] m_rewards;
    private double m_bestReward = 0.0;
    private double m_currLimit = 0;    
    private double m_multiplier = 4.0;
    private boolean m_restart = false;
    private Random m_rndm = null;
    private boolean m_rndmFlag = false;
    
    public MultiArmBandit2(boolean rndmFlag)
    {
        this.m_rndmFlag = rndmFlag;
    }
    
    public void start(int numExperts, Random rndm)
    {
        int i;         
        this.m_algo = new MABExp4();        
        this.m_rndm = rndm;
        this.m_rewards = new double[numExperts];                
        for (i=0; i<numExperts; i++) {
            this.m_rewards[i] = 0.0;
        }        
        this.m_bestReward = 0.0;
        this.m_restart = true;
    }
    
    public int chooseAction(double[][] adviceProbs) throws Exception
    {
        if (this.m_restart) {
            int numActions = adviceProbs[0].length;
            int numExperts = this.m_rewards.length;
            this.m_currLimit = ((double)numActions * Math.log((double)numExperts))/(Math.E - 1.0);
            this.m_algo.start(this.m_rewards.length, 
                              this.m_rndm); 
            this.m_restart = false;
        }
        return this.m_algo.chooseAction(adviceProbs, m_currLimit, this.m_rndmFlag);
    }
    
    public void updateReward(double reward, 
                             double[] chosenProbs) throws Exception
    {   
        if (chosenProbs.length != this.m_algo.m_numExperts) throw new Exception("wrong number of experts");
        int i;  
        double bestReward = -Double.MAX_VALUE, check;
        double[] rewards = new double[this.m_algo.m_numExperts]; 
        for (i=0; i<rewards.length; i++) {
            rewards[i] = reward * chosenProbs[i];
            this.m_rewards[i] += rewards[i];
            if (Utils.gr(this.m_rewards[i], bestReward)) {
                bestReward = this.m_rewards[i];
            }
        }                
        this.m_algo.updateReward(reward, chosenProbs); 
        // need to check if best was changed
        if (Utils.gr(bestReward, this.m_bestReward)) {                        
            this.m_bestReward = bestReward;               
            check = m_currLimit - (1.0 / this.m_algo.getAlpha());
            if (Utils.gr(bestReward, check)) {                
                m_currLimit *= m_multiplier; 
                this.m_algo.start(this.m_algo.m_numExperts, 
                                  this.m_rndm);                
            }
        }        
    }    
    
    double[] getLastActionProbs() 
    {
        return this.m_algo.m_lastActionProbArr;
    }
}
