package weka.active;

import java.util.*;
import java.io.*;
import weka.active.*;
import weka.clusterers.*;
import weka.classifiers.libsvm.*;
import weka.core.*;

public class SvmKfarInfo extends weka.active.SvmActiveLearner
                         implements RandomUser, OptionHandler
{        
    public int instanceToQuery(DataContainer pool) throws Exception
    {
        int res = super.instanceToQuery(pool);             
        if (res != -1) {
            return res;
        }
        res = this.instanceToQuerySampleMode(pool);
        if (res != -1) {   
            System.out.println("\tquerying in kfar mode.");  
            return res;                               
        }        
        throw new Exception("no res");
    }
                       
    private int instanceToQuerySampleMode(DataContainer pool) throws Exception
    {        
        int i, indx, sampleSize; 
        int trnSize = this.m_fullTrainingSet.size();
        int poolSize = pool.size();
        Instance ins;
        InstanceContainer pIns;
        
        // create sampler
        FarthestKernelSampler sampler = new FarthestKernelSampler();          
        // we will take to init all the points that were allready queried
        FastVector initSampler = new FastVector();
        FastVector sampleFrom = new FastVector();
        for (i=0; i<trnSize; i++) {
            pIns = this.m_fullTrainingSet.getInstance(i);
            initSampler.addElement(pIns);            
        }
        for (i=0; i<poolSize; i++) {
            pIns = pool.getInstance(i);
            sampleFrom.addElement(pIns);            
        }
        // sample        
        sampler.start(sampleFrom.elements(), this.m_rndm, initSampler);                
        indx = sampler.next(this.m_svm.getParams()).getIndex();
        indx = this.translateFromDataToPoolIndex(indx, pool);
        return indx;
    }
    
    public double[] distributionToQuery(DataContainer pool) throws Exception
    {
        int i; 
        int trnSize = this.m_fullTrainingSet.size();                
        InstanceContainer pIns;
        
        // create sampler
        FarthestKernelSampler sampler = new FarthestKernelSampler();          
        // we will take to init all the points that were allready queried
        FastVector initSampler = new FastVector();        
        for (i=0; i<trnSize; i++) {
            pIns = this.m_fullTrainingSet.getInstance(i);
            initSampler.addElement(pIns);
        }
        return sampler.nextDistribution(this.m_svm.getParams(), pool, initSampler);        
    }
    
    private String[] copyOptions(String[] options)
    {
        int i;                
        String[] optionsCopy = new String[options.length];
        for (i=0; i<optionsCopy.length; i++) {
            optionsCopy[i] = new String(options[i]);
        }   
        return optionsCopy;
    }
    
    public void setOptions(String options[]) throws Exception
    {
        int i;
        super.setOptions(options);       
        String SString = Utils.getOption('S', options);
        if (SString.length() != 0) {
            this.setSeed((new Integer(SString)).intValue());            
        }        
        Utils.checkForRemainingOptions(options);
    }

    public Enumeration listOptions()
    {
        FastVector newVector = new FastVector(1);        	         
        newVector.addElement(new Option(
         "\tRandom seed.\n"
         +"\t(Default is 1234567)",
         "S", 1, "-S <seed>"));
	return newVector.elements();
    }

    public String[] getOptions()
    {
       String [] options = new String [2];
       int current = 0;
       options[current++] = "-S"; options[current++] = "" + this.m_seed;       
       return options;
    }  
    
    public String toString() 
    {
        StringBuffer msg = new StringBuffer();
        msg.append("SvmSimpleCluster Active Learner\n-----------------------");
        msg.append("The SVM used:\n" + this.m_svm.toString());
        return msg.toString();
    } 
    
    public static void main(String[] args) {
        try {
            System.out.println(ActiveEvaluationPlotter.evaluateModelAndPlot(new SvmKfarInfo(), args));            
            ActiveEvaluationPlotter.Plot(m_selfTestInfo, "entropy");                   
        } catch (Exception e) {            
            System.err.println(e.getMessage());            
            e.printStackTrace();
        }  
    }        
}
