package weka.active;

import java.util.*;
import java.io.*;
import weka.active.*;
import weka.classifiers.libsvm.*;
import weka.core.*;

public class SvmRandomInfo extends weka.active.SvmActiveLearner
                           implements RandomUser, OptionHandler
{                                     
    public int instanceToQuery(DataContainer pool) throws Exception
    {
        int res = super.instanceToQuery(pool);             
        if (res != -1) {
            return res;
        }
        res = this.instanceToQueryRandomMode(pool);
        if (res != -1) {                        
            System.out.println("\tquerying in random mode.");                    
            return res;                               
        }        
        throw new Exception("no res");
    }                       
    
    public double[] distributionToQuery(DataContainer pool) throws Exception
    {
        int i;
        int size = pool.size();        
        double uniProb = 1.0/((double)size); 
        double checkSum;
        double[] prob = new double[size];
                
        checkSum = 0.0;
        for (i=0; i<size; i++) {
            prob[i] = uniProb;
            checkSum += prob[i];
        }
        // check
        if (!Utils.eq(checkSum, 1.0)) throw new Exception("sum probs is not 1.0");
        return prob;
    }
    
    private String[] copyOptions(String[] options)
    {
        int i;                
        String[] optionsCopy = new String[options.length];
        for (i=0; i<optionsCopy.length; i++) {
            optionsCopy[i] = new String(options[i]);
        }   
        return optionsCopy;
    }
    
    public void setOptions(String options[]) throws Exception
    {
        int i;
        super.setOptions(options);       
        String SString = Utils.getOption('S', options);
        if (SString.length() != 0) {
            this.setSeed((new Integer(SString)).intValue());            
        }        
        Utils.checkForRemainingOptions(options);
    }

    public Enumeration listOptions()
    {
        FastVector newVector = new FastVector(1);        	         
        newVector.addElement(new Option(
         "\tRandom seed.\n"
         +"\t(Default is 1234567)",
         "S", 1, "-S <seed>"));
	return newVector.elements();
    }

    public String[] getOptions()
    {
       String [] options = new String [2];
       int current = 0;
       options[current++] = "-S"; options[current++] = "" + this.m_seed;       
       return options;
    }  
    
    public String toString() 
    {
        StringBuffer msg = new StringBuffer();
        msg.append("SvmSimpleCluster Active Learner\n-----------------------");
        msg.append("The SVM used:\n" + this.m_svm.toString());
        return msg.toString();
    } 
    
    public static void main(String[] args) {
        try {
            System.out.println(ActiveEvaluationPlotter.evaluateModelAndPlot(new SvmRandomInfo(), args));            
            ActiveEvaluationPlotter.Plot(m_selfTestInfo, "entropy");            
        } catch (Exception e) {            
            System.err.println(e.getMessage());            
            e.printStackTrace();
        }  
    }        
}
