/* 
* Currenly, missing values are not supported! 
*/
package weka.core;

import java.util.*;
import java.io.Serializable;
import weka.core.*;

public class SparseInstance extends weka.core.Instance implements Serializable
{        
    private FastVector m_id;
    private FastVector m_val;
	
    public SparseInstance()
    {
        this.m_id = new FastVector();
        this.m_val = new FastVector();	    
    }

    public SparseInstance(int numAttributes)
    {	    
        this.m_id = new FastVector();
        this.m_val = new FastVector();
    }

    public SparseInstance(double weight, double attValues[])
    {
        this.m_Weight = weight;
        this.m_id = new FastVector();
        this.m_val = new FastVector();
        for (int i=0; i<attValues.length; i++) {
            if (!((this.m_Dataset != null) && (this.m_Dataset.classIndex()==i)) && // this is not the class value
                 (Utils.eq(attValues[i], 0) || Instance.isMissingValue(attValues[i]))) // the value is 0 or missing
            {
                continue;
            }
            this.m_id.addElement(new Integer(i));
            this.m_val.addElement(new Double(attValues[i]));
        }
        compactify();
    }
	
    public Instance makeCopy()
    {	
        SparseInstance ins = new SparseInstance();        	    
        ins.m_Weight = this.m_Weight;
        ins.m_index = this.m_index;      
        ins.m_Dataset = this.m_Dataset;
        Enumeration idEnum = this.m_id.elements();
        while (idEnum.hasMoreElements()) {
            Integer i = (Integer)idEnum.nextElement();
            ins.m_id.addElement(new Integer(i.intValue()));
        }
        Enumeration valEnum = this.m_val.elements();
        while (valEnum.hasMoreElements()) {
            Double i = (Double)valEnum.nextElement();
            ins.m_val.addElement(new Double(i.doubleValue()));
        }    
        return ins;
    }
	    
    public Object copy() {
        return this.makeCopy();
    }

    public void deleteAttributeAt(int position) throws Exception {
        if (m_Dataset != null) {
            throw new Exception("Instance has access to a dataset!");
        }
        int size = this.m_id.size();
        for (int i=0; i<size; i++) {
            if (((Integer)(this.m_id.elementAt(i))).intValue() == position) {
                this.m_id.removeElementAt(i);
                this.m_val.removeElementAt(i);
                return;
            }
        }
    }

    public void insertAttributeAt(int position) throws Exception {
        if (m_Dataset != null) {
            throw new Exception("Instance has accesss to a dataset!");
        }
        if ((position < 0) || (position > numAttributes())) {
            throw new Exception("Can't insert attribute: index out of range");
        }
        // NOP!! - no missing values are held
    }

    public boolean isMissing(int attIndex) {
        return false;
    }

    public boolean isMissing(Attribute att) {
        return this.isMissing(att.index());
    }

    public int numAttributes() {
        // if we have a dataset then its the number of attributes
        // as written there
        if (this.m_Dataset != null) {
            return this.m_Dataset.numAttributes();
        } else {
            // if not - its the highest index available
            return ((Integer)(this.m_id.elementAt(this.m_id.size()-1))).intValue()+1;
        }
    }

    public void replaceMissingValues(double[] array) throws Exception {   	 
        if ((array == null) || (array.length != m_AttValues.length)) {
            throw new Exception("Unequal number of attributes!");
        }        
        for (int i = 0; i < array.length; i++) {
            if (isMissing(i)) {
	            this.setValue(i, array[i]);
            }
        }
    }

    public void setClassMissing() throws Exception {
        if (classIndex() < 0) {
            throw new Exception("Class is not set!");
        }
        setMissing(classIndex());
    }

    public void setClassValue(double value) throws Exception {

        if (classIndex() < 0) {
            throw new Exception("Class is not set!");
        }
        this.setValue(classIndex(), value);
    }

    public void setMissing(int attIndex) throws Exception
    {
        throw new Exception("Missing values not supported in SparseInstacne");
    }
    
    public void setMissing(Attribute att) throws Exception {
        setMissing(att.index());
    }

    public void setValue(int attIndex, double value) throws Exception {        
        int i, id;        
        int size=this.m_id.size();
        boolean toEnter;
        
        if (Instance.isMissingValue(value)) {
            throw new Exception("Missing values are not supported in SparseInstance");
        }   
        toEnter = !(Utils.eq(value, 0.0));        
        for (i=0; i<size; i++) {
            id = ((Integer)(this.m_id.elementAt(i))).intValue();
            if (id == attIndex) {
                if (toEnter) {
                    this.m_val.setElementAt(new Double(value), i);                    
                } else {
                    this.m_val.removeElementAt(i);
                    this.m_id.removeElementAt(i);                                        
                }
                return;
            }    
            if ((id > attIndex) && toEnter) {
                this.m_id.insertElementAt(new Integer(attIndex), i);
                this.m_val.insertElementAt(new Double(value), i); 
                return;
            }
        }
        if (toEnter) {
            this.m_id.addElement(new Integer(attIndex));
            this.m_val.addElement(new Double(value));
        }
    }

    public void setValue(int attIndex, String value) throws Exception {        
        int valIndex;
        if (m_Dataset == null) {
            throw new Exception("Instance doesn't have access to a dataset!");
        }
        if (!attribute(attIndex).isNominal() && !attribute(attIndex).isString()) {
            throw new Exception("Attribute neither nominal nor string!");
        }
        valIndex = attribute(attIndex).indexOfValue(value);
        if (valIndex == -1) {
            if (attribute(attIndex).isNominal()) {
	            throw new Exception("Value not defined for given nominal attribute!");
            } else {
	            attribute(attIndex).forceAddValue(value);
	            valIndex = attribute(attIndex).indexOfValue(value);
            }
        }
        this.setValue(attIndex, (double)valIndex);        
    }

    public void setValue(Attribute att, double value) throws Exception {
        this.setValue(att.index(), (double)value);
    }

    public void setValue(Attribute att, String value) throws Exception {
        if (!att.isNominal() && !att.isString()) {
            throw new Exception("Attribute neither nominal nor string!");
        }
        int valIndex = att.indexOfValue(value);
        if (valIndex == -1) {
            if (att.isNominal()) {
	            throw new Exception("Value not defined for given nominal attribute!");
            } else {
	            att.forceAddValue(value);
	            valIndex = att.indexOfValue(value);
            }
        }
        this.setValue(att.index(), (double)valIndex);
    }

    public String stringValue(int attIndex) throws Exception {
        if (m_Dataset == null) {
            throw new Exception("Instance doesn't have access to a dataset!");
        } 
        if (!m_Dataset.attribute(attIndex).isNominal() &&
	        !m_Dataset.attribute(attIndex).isString()) {
            throw new Exception("Attribute neither nominal nor string!");
        }        
        return m_Dataset.attribute(attIndex).value((int)this.findValue(attIndex));
    }

    public String stringValue(Attribute att) throws Exception {
        return this.stringValue(att.index());
    }

    public String toString() {
        StringBuffer text = new StringBuffer();
        int size = this.m_id.size();
        Integer indx;
        Double value;
        for (int i = 0; i < size; i++) {
            if (i > 0) text.append(",");
            indx = (Integer)(this.m_id.elementAt(i));
            value = (Double)(this.m_val.elementAt(i));
            text.append("" + indx.intValue() + ":" + Utils.doubleToString(value.doubleValue(),6));                       
        }
        return text.toString();
    }
    
    public void toStringBuff(StringBuffer text) 
    {        
        int size = this.m_id.size();
        Integer indx;
        Double value;
        for (int i = 0; i < size; i++) {
            if (i > 0) text.append(",");
            indx = (Integer)(this.m_id.elementAt(i));
            value = (Double)(this.m_val.elementAt(i));
            text.append("" + indx.intValue() + ":" + Utils.doubleToString(value.doubleValue(),6));                       
        }
    }
    
    public void toStringBuffSvmLight(StringBuffer text) throws Exception 
    {        
        int size = this.m_id.size();
        Integer indx;
        Double value;
        int clsIndx = this.classIndex();
        int clsValue = (int)(this.value(clsIndx));
        if (clsValue == 0) {
            text.append("-1");
        } else {
            if (clsValue != 1) throw new Exception("class value is not 0 and not 1");
            text.append("1");
        }
        for (int i=0; i<size; i++) {            
            indx = (Integer)(this.m_id.elementAt(i));
            if (indx.intValue() == clsIndx) continue;
            value = (Double)(this.m_val.elementAt(i));
            text.append(" " + (indx.intValue()+1) + ":" + Utils.doubleToString(value.doubleValue(),6));
        }
        text.append("\n");
    }
    
    public String toString(int attIndex) 
    {        
        StringBuffer text = new StringBuffer();       
        if (isMissing(attIndex)) {
            text.append("?");
        } else {
            if (m_Dataset == null) {
                text.append(Utils.doubleToString(this.findValue(attIndex),6));
            } else {
                if (m_Dataset.attribute(attIndex).isNominal() || 
	                m_Dataset.attribute(attIndex).isString()) {
	                try {
	                    text.append(Utils.quote(stringValue(attIndex)));
	                } catch (Exception e) {
    	                throw new Error("This should never happen!");
	                }
                } else {
	                text.append(Utils.doubleToString(this.findValue(attIndex),6));
                }
            }
        }
        return text.toString();
    }
    
    public String toString(Attribute att) {       
        return toString(att.index());
    }

    public double value(int attIndex) {
        double val = this.findValue(attIndex);        
        if (Instance.isMissingValue(val)) {
            val = 0.0;
        }        
        return val;
    }

    public double value(Attribute att) {
        return this.value(att.index());
    }
    
    public void compactify()
    {
        this.m_id.trimToSize();
        this.m_val.trimToSize();
    }
    
    private double findValue(int attIndex)
    {
        int i, id;
        int size = this.m_id.size();    
        if (attIndex+1 <size) {
            size = attIndex+1;
        }
        for (i=0; i<size; i++) {
            id=((Integer)(this.m_id.elementAt(i))).intValue();
            if (id == attIndex) {
                return ((Double)(this.m_val.elementAt(i))).doubleValue();
            }                           
        }
        if ((this.m_Dataset!=null) && (this.m_Dataset.classIndex()==attIndex)) {
            return Instance.missingValue();
        }
        return 0;
    }
    
    public FastVector values() throws Exception
    {
        return this.m_val;
    }
    
    public FastVector ids() throws Exception
    {
        return this.m_id;
    }
    
    public void setIdsVals(FastVector ids, FastVector vals) throws Exception
    {
        if (ids.size() != vals.size()) {
            throw new Exception("SparseInstance: ids and vals must be in the same length");
        }
        this.m_id = ids;
        this.m_val = vals;
        this.compactify();
    }
        
    public static Object[] mergeAndSort2Instances(SparseInstance s1,
                                                  SparseInstance s2) throws Exception
    {        
        int indx = -1, lastId = -1, itr = 0;
        int classIndx;
        int[] currId = new int[2];
        FastVector[] idVec = new FastVector[2];
        FastVector[] valVec = new FastVector[2];
        Enumeration[] idEnum = new Enumeration[2];
        Enumeration[] valEnum = new Enumeration[2];       
        valVec[0] = s1.values(); valEnum[0] = valVec[0].elements();
        idVec[0] = s1.ids(); idEnum[0] = idVec[0].elements();
        valVec[1] = s2.values(); valEnum[1] = valVec[1].elements();
        idVec[1] = s2.ids(); idEnum[1] = idVec[1].elements();        
        int totalSize = idVec[0].size() + idVec[1].size() + 1; // one more for marking the end
        try {
            classIndx = s1.classIndex(); 
        } catch (Exception e) {    
            classIndx = s2.classIndex();
        }
        int[] ids = new int[totalSize];
        double[][] vals = new double[2][];
        for (indx=0; indx<2; indx++) {
            vals[indx] = new double[totalSize];            
        }
      
        while (idEnum[0].hasMoreElements() || idEnum[1].hasMoreElements()) {
            if (lastId == -1) { 
                // indx and lastId values are invalid                                
                if (idEnum[0].hasMoreElements()) {
                    currId[0] = ((Integer)idEnum[0].nextElement()).intValue();
                    if (currId[0] == classIndx) {
                        if (idEnum[0].hasMoreElements()) {                        
                            currId[0] = ((Integer)idEnum[0].nextElement()).intValue();
                            valEnum[0].nextElement();
                        } else {
                             // no more values
                            currId[0] = Integer.MAX_VALUE;
                        }
                    }
                } else {
                    // no more values
                    currId[0] = Integer.MAX_VALUE;
                }                
                if (idEnum[1].hasMoreElements()) {
                    currId[1] = ((Integer)idEnum[1].nextElement()).intValue();  
                    if (currId[1] == classIndx) {
                        if (idEnum[1].hasMoreElements()) {                        
                            currId[1] = ((Integer)idEnum[1].nextElement()).intValue();
                            valEnum[1].nextElement();
                        } else {
                            if (currId[0] == Integer.MAX_VALUE) continue;
                            currId[1] = Integer.MAX_VALUE;                                       
                        }
                    }                    
                } else {
                    if (currId[0] == Integer.MAX_VALUE) continue;
                    currId[1] = Integer.MAX_VALUE;            
                }
            } else {
                // indx holds the array that we are running on currently
                // lastId is the smallest value in the second array   
                if (idEnum[indx].hasMoreElements()) {                
                    currId[indx] = ((Integer)idEnum[indx].nextElement()).intValue();
                    if (currId[indx] == classIndx) {
                        if (idEnum[indx].hasMoreElements()) {                        
                            currId[indx] = ((Integer)idEnum[indx].nextElement()).intValue();
                            valEnum[indx].nextElement();
                        } else {
                            if (currId[(indx+1)%2] == Integer.MAX_VALUE) continue;                    
                            currId[indx] = Integer.MAX_VALUE;                            
                        }
                    }                    
                } else {
                    if (currId[(indx+1)%2] == Integer.MAX_VALUE) continue;                    
                    currId[indx] = Integer.MAX_VALUE;
                }
                currId[(indx+1)%2] = lastId;                
            }
            if (currId[0] == currId[1]) { 
                ids[itr] = currId[0];                                
                for (indx=0; indx<2; indx++) {                    
                    vals[indx][itr] = ((Double)valEnum[indx].nextElement()).doubleValue();                    
                }                 
                lastId = -1;
            } else {
                indx = (currId[0] < currId[1]) ? 0 : 1;                
                ids[itr] = currId[indx];
                vals[indx][itr] = ((Double)valEnum[indx].nextElement()).doubleValue(); 
                vals[(indx+1)%2][itr] = 0;                                
                lastId = currId[(indx+1)%2];                
            }        
            itr++;
        } 
        // mark the end
        ids[itr] = -1;            
        Object[] ans = new Object[2];
        ans[0] = ids;
        ans[1] = vals;
        return ans;
    }
}